
//
//  PQBaseWebViewController.swift
//  PQSpeed
//
//  Created by SanW on 2020/5/27.
//  Copyright © 2020 BytesFlow. All rights reserved.
//

import UIKit
import WebKit

open class PQBaseWebViewController: PQBaseViewController {
    public var emptyData: PQEmptyModel? = {
        let emptyData = PQEmptyModel()
        emptyData.title = "网页加载失败，请重试～"
        emptyData.emptyImageName = "pic_network"
        emptyData.netDisRefreshBgColor = UIColor.hexColor(hexadecimal: "#FA6400")
        emptyData.netDisTitle = "内容加载失败"
        emptyData.netDisTitleColor = UIColor.hexColor(hexadecimal: "#333333")
        emptyData.netemptyDisImage = UIImage.init(named: "empty_netDis_icon")
        emptyData.netDisRefreshTitle = NSMutableAttributedString(string: "重新加载", attributes: [.font: UIFont.systemFont(ofSize: 16, weight: .medium), .foregroundColor: UIColor.white])
        return emptyData
    }()

    public lazy var emptyRemindView: PQEmptyRemindView = {
        let emptyRemindView = PQEmptyRemindView(frame: CGRect(x: 0, y: cDevice_iPhoneNavBarAndStatusBarHei, width: cScreenWidth, height: cScreenHeigth - cDevice_iPhoneNavBarAndStatusBarHei))
        emptyRemindView.isHidden = true
        emptyRemindView.fullRefreshBloc = { [weak self] _, _ in
            self?.refreshClick()
        }
        emptyRemindView.emptyData = emptyData
        return emptyRemindView
    }()

    public lazy var webView: WKWebView = {
        let config: WKWebViewConfiguration = WKWebViewConfiguration()
        config.allowsInlineMediaPlayback = true
        let webView: WKWebView = WKWebView(frame: CGRect(x: 0, y: cDevice_iPhoneNavBarAndStatusBarHei, width: cScreenWidth, height: cScreenHeigth - cDevice_iPhoneNavBarAndStatusBarHei), configuration: config)
        webView.backgroundColor = UIColor.white
        if #available(iOS 11.0, *) {
            webView.scrollView.contentInsetAdjustmentBehavior = .never
        } else {
            automaticallyAdjustsScrollViewInsets = false
        }
        webView.navigationDelegate = self
        return webView
    }()

    public lazy var progresslayer: CALayer = {
        let progresslayer = CALayer()
        progresslayer.frame = CGRect(x: 0, y: cDevice_iPhoneNavBarAndStatusBarHei, width: cScreenWidth * 0.1, height: 2)
        progresslayer.backgroundColor = UIColor.hexColor(hexadecimal: "#FF9500").cgColor
        return progresslayer
    }()

    @objc public var baseUrl: String? {
        didSet {
            if baseUrl != nil, baseUrl?.count ?? 0 > 0 {
                webView.load(URLRequest(url: NSURL(string: baseUrl ?? "")! as URL, cachePolicy: .reloadIgnoringCacheData))
                // 添加属性监听
                webView.addObserver(self, forKeyPath: "estimatedProgress", options: .new, context: nil)
                isAddObserve = true
                view.layer.addSublayer(progresslayer)
            }
        }
    }

    @objc public var request: URLRequest? {
        didSet {
            if request != nil, request?.url != nil {
                webView.load(request!)
                // 添加属性监听
                webView.addObserver(self, forKeyPath: "estimatedProgress", options: .new, context: nil)
                isAddObserve = true
                view.layer.addSublayer(progresslayer)
            }
        }
    }

    public var deleteBtn: UIButton = {
        let deleteBtn = UIButton(type: .custom)
        deleteBtn.frame = CGRect(x: 0, y: cDevice_iPhoneStatusBarHei, width: cDefaultMargin * 4, height: cDefaultMargin * 4)
        deleteBtn.imageEdgeInsets = UIEdgeInsets(top: 0, left: 0, bottom: -5, right: 0)
        deleteBtn.tintColor = UIColor.black
        deleteBtn.setImage(UIImage(named: "upload_delete")?.withRenderingMode(.alwaysTemplate), for: .normal)
        deleteBtn.addTarget(self, action: #selector(clickHandle), for: .touchUpInside)
        deleteBtn.isHidden = true
        return deleteBtn
    }()

    @objc public var baseTitle: String?
    public var isShowDelete: Bool = false {
        didSet {
            deleteBtn.isHidden = !isShowDelete
        }
    }

    public var evaluateJavaScript: String? // 交互
    public var isAddObserve: Bool = false

    override open func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view.
        view.addSubview(webView)
        view.addSubview(emptyRemindView)
        leftButton(imageName: "icon_blanc_back")
        backButton?.tintColor = UIColor.black
//        backButton?.setImage(UIImage(named: "icon_blanc_back")?.withRenderingMode(.alwaysTemplate), for: .normal)
        navHeadImageView?.backgroundColor = UIColor.white
        navHeadImageView?.addSubview(deleteBtn)
        deleteBtn.frame.origin.x = (backButton?.frame.maxX ?? 0)
        // 拦截侧滑返回
//        disablePopGesture().popGestureHandle = { [weak self] in
//            self?.back()
//        }
    }

    override public func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        if (UIApplication.shared.keyWindow?.viewWithTag(cProtocalViewTag)) != nil {
            (UIApplication.shared.keyWindow?.viewWithTag(cProtocalViewTag))?.isHidden = true
        }
    }

    override public func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        if (UIApplication.shared.keyWindow?.viewWithTag(cProtocalViewTag)) != nil {
            (UIApplication.shared.keyWindow?.viewWithTag(cProtocalViewTag))?.isHidden = false
        }
    }

    override open func backBtnClick() {
        back()
    }

    deinit {
        if isAddObserve {
            webView.removeObserver(self, forKeyPath: "estimatedProgress")
        }
    }
}

extension PQBaseWebViewController: WKNavigationDelegate {
    public func refreshClick() {
        if baseUrl != nil, baseUrl?.count ?? 0 > 0 {
            webView.load(URLRequest(url: NSURL(string: baseUrl ?? "")! as URL, cachePolicy: .useProtocolCachePolicy))
        } else if request != nil, request?.url != nil {
            webView.load(request!)
        }
    }

    @objc public func clickHandle() {
        if navigationController != nil {
            navigationController?.popViewController(animated: true)
        } else {
            dismiss(animated: true, completion: nil)
        }
    }

    @objc open func back() {
        if webView.canGoBack {
            webView.goBack()
        } else if navigationController != nil {
            navigationController?.popViewController(animated: true)
        } else {
            dismiss(animated: true, completion: nil)
        }
    }

    override public func observeValue(forKeyPath keyPath: String?, of object: Any?, change: [NSKeyValueChangeKey: Any]?, context: UnsafeMutableRawPointer?) {
        if keyPath == "estimatedProgress" {
            progresslayer.opacity = 1
            let float = (change?[NSKeyValueChangeKey.newKey] as! NSNumber).floatValue
            progresslayer.frame = CGRect(x: 0, y: cDevice_iPhoneNavBarAndStatusBarHei, width: cScreenWidth * CGFloat(float), height: 3)
            if float == 1 {
                weak var weakself = self
                DispatchQueue.main.asyncAfter(deadline: DispatchTime.now() + 0.2) {
                    weakself?.progresslayer.opacity = 0
                }
                DispatchQueue.main.asyncAfter(deadline: DispatchTime.now() + 0.8) {
                    weakself?.progresslayer.frame = CGRect(x: 0, y: cDevice_iPhoneNavBarAndStatusBarHei, width: 0, height: 3)
                }
            }
        } else {
            super.observeValue(forKeyPath: keyPath, of: object, change: change, context: context)
        }
    }

    public func webView(_ webView: WKWebView, didFinish _: WKNavigation!) {
        debugPrint("webView回调-didFinish")
//        if baseTitle == nil || baseTitle?.count ?? 0 <= 0 {
        webView.evaluateJavaScript("document.title") { [weak self] (any, _) -> Void in
            self?.setTitle(title: any as? String, color: UIColor.black)
            self?.baseTitle = any as? String
        }
//        }
        if evaluateJavaScript != nil, (evaluateJavaScript?.count ?? 0) > 0 {
            webView.evaluateJavaScript(evaluateJavaScript!) { _, _ in
            }
        }
        if !emptyRemindView.isHidden {
            emptyRemindView.isHidden = true
        }
    }

    public func webView(_ webView: WKWebView, didFail _: WKNavigation!, withError error: Error) {
        debugPrint("webView回调-didFailProvisionalNavigation -error:\(error)")
//        if baseTitle == nil || baseTitle?.count ?? 0 <= 0 {
        webView.evaluateJavaScript("document.title") { [weak self] (any, _) -> Void in
            self?.setTitle(title: any as? String, color: UIColor.black)
            self?.baseTitle = any as? String
        }
//        }
        if emptyRemindView.isHidden {
            emptyRemindView.isHidden = false
        }
    }

    public func webView(_: WKWebView, didFailProvisionalNavigation _: WKNavigation!, withError error: Error) {
        debugPrint("webView回调-didFailProvisionalNavigation -error:\(error)")
//        if emptyRemindView.isHidden {
//            emptyRemindView.isHidden = false
//        }
    }

    public func webView(_: WKWebView, decidePolicyFor navigationResponse: WKNavigationResponse, decisionHandler: @escaping (WKNavigationResponsePolicy) -> Void) {
        debugPrint("navigationResponse:\(String(describing: navigationResponse))")
        webView.evaluateJavaScript("document.title") { [weak self] (any, _) -> Void in
            self?.setTitle(title: any as? String, color: UIColor.black)
            self?.baseTitle = any as? String
        }
        decisionHandler(.allow)
    }

    public func webView(_: WKWebView, didStartProvisionalNavigation navigation: WKNavigation!) {
        debugPrint("didStartProvisionalNavigation:\(String(describing: navigation))")
    }

    public func webView(_: WKWebView, didReceiveServerRedirectForProvisionalNavigation navigation: WKNavigation!) {
        debugPrint("didReceiveServerRedirectForProvisionalNavigation:\(String(describing: navigation))")
    }

    public func webView(_: WKWebView, didCommit navigation: WKNavigation!) {
        debugPrint("\(String(describing: navigation))")
    }

    public func webView(_: WKWebView, decidePolicyFor navigationAction: WKNavigationAction, decisionHandler: @escaping (WKNavigationActionPolicy) -> Void) {
        let urlString: String = navigationAction.request.url?.absoluteString ?? ""
        let url: URL? = URL(string: urlString)
//        if urlString.count == 0 || urlString == "about:blank" {
//            decisionHandler(.cancel)
//            return
//        }
        if urlString.hasPrefix("weixin://"), url != nil {
            let canOpenUrl = UIApplication.shared.canOpenURL(url!)
            if canOpenUrl {
                UIApplication.shared.open(url!, options: [:], completionHandler: nil)
                decisionHandler(.allow)
            } else {
                decisionHandler(.cancel)
            }
        } else {
            decisionHandler(.allow)
        }

//        let vc = PQBaseWebViewController.init()
//        vc.baseUrl = url
//        navigationController?.pushViewController(vc, animated: true)
//        PQLog(message: "decidePolicyFor \(String(describing: navigationAction))")
//        decisionHandler(.allow)
    }

    override public var preferredStatusBarStyle: UIStatusBarStyle {
        if #available(iOS 13.0, *) {
            return .darkContent
        } else {
            // Fallback on earlier versions
            return .default
        }
    }
}
